package com.Vente;

/**
 * Classe Produit<T> : un produit est composé d'une chose vendable et d'une quantité.
 * Utilise la généricité pour permettre la compatibilité avec n'importe quel type de Vendable.
 */
public class Produit {
    private Vendable chose;
    private int quantite;

    public Produit(Vendable chose, int quantite) {
        if (quantite <= 0) {
            throw new IllegalArgumentException("La quantité doit être strictement positive.");
        }
        this.chose = chose;
        this.quantite = quantite;

        assert checkInvariant() : "Invariant violé : Produit invalide.";	
    }

    /**
     * Retourne le prix total du produit (quantité * prix unitaire).
     */
    public double getPrixTotal() {
        double prixTotal = quantite * chose.getPrixUnitaire();
        assert prixTotal >= 0 : "Postcondition violée : Prix total négatif.";
        return prixTotal;
    }

    /**
     * Vérifie l'invariant de la classe Produit.
     * La quantité doit toujours être > 0.
     */
    private boolean checkInvariant() {
        return quantite > 0;
    }
}